@@ -1000,9 +1000,7 @@ public abstract class WsRemoteEndpointImplBase implements RemoteEndpoint {
 
         @Override
         public void write(int b) throws IOException {
-            if (closed) {
-                throw new IllegalStateException(sm.getString("wsRemoteEndpoint.closedOutputStream"));
-            }
+            checkOpen();
 
             used = true;
             if (buffer.remaining() == 0) {
@@ -1013,9 +1011,7 @@ public abstract class WsRemoteEndpointImplBase implements RemoteEndpoint {
 
         @Override
         public void write(byte[] b, int off, int len) throws IOException {
-            if (closed) {
-                throw new IllegalStateException(sm.getString("wsRemoteEndpoint.closedOutputStream"));
-            }
+            checkOpen();
             if ((off < 0) || (off > b.length) || (len < 0) || ((off + len) > b.length) || ((off + len) < 0)) {
                 throw new IndexOutOfBoundsException();
             }
@@ -1043,9 +1039,7 @@ public abstract class WsRemoteEndpointImplBase implements RemoteEndpoint {
 
         @Override
         public void flush() throws IOException {
-            if (closed) {
-                throw new IllegalStateException(sm.getString("wsRemoteEndpoint.closedOutputStream"));
-            }
+            checkOpen();
 
             // Optimisation. If there is no data to flush then do not send an
             // empty message.
@@ -1066,6 +1060,12 @@ public abstract class WsRemoteEndpointImplBase implements RemoteEndpoint {
             doWrite(true);
         }
 
+        private void checkOpen() throws IOException {
+            if (closed) {
+                throw new IOException(sm.getString("wsRemoteEndpoint.closedOutputStream"));
+            }
+        }
+
         private void doWrite(boolean last) throws IOException {
             if (used) {
                 buffer.flip();
@@ -1091,9 +1091,7 @@ public abstract class WsRemoteEndpointImplBase implements RemoteEndpoint {
 
         @Override
         public void write(char[] cbuf, int off, int len) throws IOException {
-            if (closed) {
-                throw new IllegalStateException(sm.getString("wsRemoteEndpoint.closedWriter"));
-            }
+            checkOpen();
             if ((off < 0) || (off > cbuf.length) || (len < 0) || ((off + len) > cbuf.length) || ((off + len) < 0)) {
                 throw new IndexOutOfBoundsException();
             }
@@ -1121,9 +1119,7 @@ public abstract class WsRemoteEndpointImplBase implements RemoteEndpoint {
 
         @Override
         public void flush() throws IOException {
-            if (closed) {
-                throw new IllegalStateException(sm.getString("wsRemoteEndpoint.closedWriter"));
-            }
+            checkOpen();
 
             if (buffer.position() > 0) {
                 doWrite(false);
@@ -1142,6 +1138,12 @@ public abstract class WsRemoteEndpointImplBase implements RemoteEndpoint {
             doWrite(true);
         }
 
+        private void checkOpen() throws IOException {
+            if (closed) {
+                throw new IOException(sm.getString("wsRemoteEndpoint.closedWriter"));
+            }
+        }
+
         private void doWrite(boolean last) throws IOException {
             if (used) {
                 buffer.flip();
